package tsyliveaudiorecord.live_audio_record;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.os.Binder;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.util.Log;

import androidx.annotation.RequiresApi;
import androidx.core.app.NotificationCompat;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.Locale;
import java.util.Queue;

import io.reactivex.disposables.Disposable;
import io.reactivex.functions.Consumer;

public class AudioRecordService extends Service {

    private static final String LOG_TAG = "RecordingService";

    int mSampleRate = 16000;

    public AudioRecorder audioRecorder;
    private NotificationManager mNotificationManager;
    private static final int NOTIFY_ID = 100;

    public  AudioSaveHelper audioSaveHelper;
    public AudioRecordingDbmHandler handler;
    private ServiceBinder mIBinder;
    private AudioRecorder.RecordTime lastUpdated;
    private boolean mIsClientBound = false;
    int sampleRate = 16000;
    String filePath = "2222";
    String mExtension ="" ;
    int bufferSize = 3000 ;


    @Override public IBinder onBind(Intent intent) {
        mIsClientBound = true;
        return mIBinder;
    }

    public boolean isRecording() {
        return audioRecorder.isRecording();
    }

    @Override
    public void onCreate() {
        super.onCreate();
        Log.d("kiemtra","hello");

        handler = new AudioRecordingDbmHandler();
        audioSaveHelper = new AudioSaveHelper();

        audioRecorder = new AudioRecorder(audioSaveHelper);
        handler.addRecorder(audioRecorder);

        mIBinder = new ServiceBinder();
        mNotificationManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);

    }



    public AudioRecordingDbmHandler getHandler() {
        return handler;
    }
    Handler timerHandler;
    Runnable timerRunnable;
    @Override public int onStartCommand(Intent intent, int flags, int startId) {
        mSampleRate = intent.getIntExtra("mSampleRate",16000);

        if (intent.getAction() != null) {
            switch (intent.getAction()) {
                case AppConstants.ACTION_PAUSE:
                    pauseRecord();
                    break;
                case AppConstants.ACTION_RESUME:
                    resumeRecord();
                    break;
                case AppConstants.ACTION_STOP:
                    if (!mIsClientBound) {
                        stopSelf();
                    }
            }
            if (mIsClientBound) {
                intent.putExtra(AppConstants.ACTION_IN_SERVICE, intent.getAction());
                intent.setAction(AppConstants.ACTION_IN_SERVICE);
                LocalBroadcastManager.getInstance(this).sendBroadcast(intent);
            }
        }
        else {
            filePath = intent.getStringExtra("mFilePath");
            sampleRate = intent.getIntExtra("mSampleRate",16000);
            startRecording();
           timerHandler = new Handler();
           timerRunnable  = new Runnable() {

               @Override
               public void run() {
                   HashMap<String, Object> x  = audioRecorder.qe.poll();
                   if(x != null) {
                       Intent intent = new Intent("GPSLocationUpdates");
                       // You can also include some extra data.
                       intent.putExtra("current", x);
                       LocalBroadcastManager.getInstance(getApplicationContext()).sendBroadcast(intent);
                   }

                   timerHandler.postDelayed(this, 32);
               }
           };

            timerRunnable.run();

            startForeground(101, createNotification(new AudioRecorder.RecordTime()));
        }
        return START_NOT_STICKY;
    }
    @RequiresApi(Build.VERSION_CODES.O)
    private String createNotificationChannel(String channelId, String channelName){
        NotificationChannel chan = new NotificationChannel(channelId,
                channelName, NotificationManager.IMPORTANCE_NONE);
        chan.setLightColor(Color.BLUE);
        chan.setLockscreenVisibility(Notification.VISIBILITY_PRIVATE);
        mNotificationManager.createNotificationChannel(chan);
        return channelId;
    }




    @Override public void onDestroy() {
        super.onDestroy();
        if (isRecording()) {
            stopRecodingAndRelease();
            timerHandler.removeCallbacks(timerRunnable);
        }
    }

    private void stopRecodingAndRelease() {
        audioRecorder.finishRecord();
        timerHandler.removeCallbacks(timerRunnable);

        handler.stop();
    }

    private void startRecording() {
        audioRecorder.startRecord(
                mSampleRate ,filePath);
        handler.startDbmThread();
//        audioRecorder.subscribeTimer(this::updateNotification);
    }

    public Disposable subscribeForTimer(Consumer<AudioRecorder.RecordTime> timerConsumer) {
        return audioRecorder.subscribeTimer(timerConsumer);
    }

    private void updateNotification(AudioRecorder.RecordTime recordTime) {
        mNotificationManager.notify(NOTIFY_ID, createNotification(recordTime));
    }
    private Notification createNotification(AudioRecorder.RecordTime recordTime) {
        lastUpdated = recordTime;
        String channeID = "" ;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O){
            channeID = createNotificationChannel("tsyliveaudiorecord.live_audio_record","My Background Service");
        }
        NotificationCompat.Builder notificationBuilder = new NotificationCompat.Builder(this, channeID );
        Notification notification = notificationBuilder.setOngoing(true)
                .setSmallIcon(R.drawable.ic_media_record)
                .setContentTitle(getString(R.string.notification_recording))
                .setContentText("Recording...........")
//                .addAction(R.drawable.ic_media_stop, getString(R.string.stop_recording),
//                        getActionIntent(AppConstants.ACTION_STOP))
                .setPriority(NotificationManager.IMPORTANCE_MIN)
                .setCategory(Notification.CATEGORY_SERVICE)
                .build();
//        if (audioRecorder.isPaused()) {
//            notificationBuilder.addAction(R.drawable.ic_media_record, getString(R.string.resume_recording_button),
//                    getActionIntent(AppConstants.ACTION_RESUME));
//        } else {
//            notificationBuilder.addAction(R.drawable.ic_media_pause, getString(R.string.pause_recording_button),
//                    getActionIntent(AppConstants.ACTION_PAUSE));
//        }
//    mBuilder.setContentIntent(PendingIntent.getActivities(getApplicationContext(), 0,
//        new Intent[] {new Intent(getApplicationContext(), MainActivity.class)}, 0));

        return notification;
    }


    public void pauseRecord() {
        audioRecorder.pauseRecord();

    }
    public boolean isPaused() {
        return audioRecorder.isPaused();
    }

    public void resumeRecord() {
        audioRecorder.resumeRecord();
    }

    public class ServiceBinder extends Binder {
        public AudioRecordService getService() {
            return AudioRecordService.this;
        }
    }

    private PendingIntent getActionIntent(String action) {
        Intent pauseIntent = new Intent(this, AudioRecordService.class);
        pauseIntent.setAction(action);
        return PendingIntent.getService(this, 100, pauseIntent, 0);
    }

    @Override public boolean onUnbind(Intent intent) {
        mIsClientBound = false;
        return true;
    }

    @Override public void onRebind(Intent intent) {
        mIsClientBound = true;
    }
}
