package tsyliveaudiorecord.live_audio_record;


import android.Manifest;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.ServiceConnection;
import android.content.pm.PackageManager;
import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.MediaRecorder;

import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;

import android.os.IBinder;
import android.util.Log;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.HashMap;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.Queue;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import io.reactivex.disposables.Disposable;
import io.reactivex.functions.Consumer;

/** LiveAudioRecordPlugin */
public class VaisSpeechPlugin implements MethodCallHandler, PluginRegistry.RequestPermissionsResultListener {
  private static final String LOG_NAME = "AndroidAudioRecorder";
  private static final int PERMISSIONS_REQUEST_RECORD_AUDIO = 200;
  private static final byte RECORDER_BPP = 16; // we use 16bit
  private Registrar registrar;
  private int mSampleRate = 16000; // 16Khz
  private String mFilePath;
  private String mExtension;
  private int bufferSize = 1024;
  private FileOutputStream mFileOutputStream = null;
  private String mStatus = "unset";

  private long mDataSize = 0;
  private Result _result;
  private byte[] tempByteBuffer;
  Queue<HashMap<String, Object>> qe = new LinkedList();
  Intent intent = null;
  AudioRecordService audioRecordService;


  /** Plugin registration. */
  public static void registerWith(Registrar registrar) {

    final MethodChannel channel = new MethodChannel(registrar.messenger(), "flutter_audio_recorder");
    channel.setMethodCallHandler(new VaisSpeechPlugin(registrar));
  }

  public VaisSpeechPlugin(Registrar registrar) {
    this.registrar = registrar;
    this.registrar.addRequestPermissionsResultListener(this);
    audioRecordService = new AudioRecordService();
     intent = new Intent(registrar.context(), audioRecordService.getClass());

  }

  @Override
  public boolean onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
    final int REQUEST_RECORD_AUDIO_PERMISSION = 200;
    switch (requestCode) {
      case REQUEST_RECORD_AUDIO_PERMISSION:
        boolean granted = true;
        Log.d(LOG_NAME, "parsing result");
        for (int result : grantResults) {
          if(result != PackageManager.PERMISSION_GRANTED) {
            Log.d(LOG_NAME, "result" + result);
            granted = false;
          }
        }
        Log.d(LOG_NAME, "onRequestPermissionsResult -" + granted);
        if(_result != null) {
          _result.success(granted);
        }
        return granted;
      default:
        Log.d(LOG_NAME, "onRequestPermissionsResult - false");
        return false;
    }
  }

  private boolean hasRecordPermission(){
    // if after [Marshmallow], we need to check permission on runtime
    if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.M) {
      return (ContextCompat.checkSelfPermission(registrar.context(), Manifest.permission.RECORD_AUDIO) == PackageManager.PERMISSION_GRANTED)
              && (ContextCompat.checkSelfPermission(registrar.context(), Manifest.permission.WRITE_EXTERNAL_STORAGE) == PackageManager.PERMISSION_GRANTED);
    } else {
      return ContextCompat.checkSelfPermission(registrar.context(), Manifest.permission.RECORD_AUDIO) == PackageManager.PERMISSION_GRANTED;
    }
  }

  @Override
  public void onMethodCall(MethodCall call, Result result) {
    // Log.d(LOG_NAME, "calling " + call.method);
    _result = result;

    switch (call.method){
      case "hasPermissions":
        handleHasPermission();
        break;
      case "init":
        handleInit(call, result);
        break;
      case "current":
        handleCurrent(call, result);
        break;
      case "start":
        handleStart(call, result);
        break;
      case "pause":
        handlePause(call, result);
        break;
      case "resume":
        handleResume(call, result);
        break;
      case "stop":
        handleStop(call, result);
        break;
      default:
        result.notImplemented();
        break;
    }
  }

  private void handleHasPermission(){
    if(hasRecordPermission()){
      Log.d(LOG_NAME, "handleHasPermission true");
      if(_result != null) {
        _result.success(true);
      }
    } else {
      Log.d(LOG_NAME, "handleHasPermission false");

      if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.M) {
        ActivityCompat.requestPermissions(registrar.activity(), new String[]{Manifest.permission.RECORD_AUDIO, Manifest.permission.WRITE_EXTERNAL_STORAGE}, PERMISSIONS_REQUEST_RECORD_AUDIO);
      } else {
        ActivityCompat.requestPermissions(registrar.activity(), new String[]{Manifest.permission.RECORD_AUDIO}, PERMISSIONS_REQUEST_RECORD_AUDIO);
      }
    }

  }

  private void handleInit(MethodCall call, Result result)  {
    mSampleRate = Integer.parseInt(call.argument("sampleRate").toString());
    mFilePath = call.argument("path").toString();
    mExtension = call.argument("extension").toString();
    bufferSize = AudioRecord.getMinBufferSize(mSampleRate, AudioFormat.CHANNEL_IN_MONO, AudioFormat.ENCODING_PCM_16BIT);
    mStatus = "initialized";
    HashMap<String, Object> initResult = new HashMap<>();
    initResult.put("duration", 0);
    initResult.put("path", mFilePath);
    initResult.put("audioFormat", mExtension);
    initResult.put("peakPower", -120.00);
    initResult.put("averagePower", -120.00);
    initResult.put("isMeteringEnabled", true);
    initResult.put("status", mStatus);
    bindToService();
    result.success(initResult);
  }

  private void handleCurrent(MethodCall call, Result result) {
    HashMap<String, Object> current = qe.poll();
    if(current!=null){
      result.success(current);
    }
  }

  private void handleStart(MethodCall call, Result result) {
    LocalBroadcastManager.getInstance(registrar.activity()).registerReceiver(
            mMessageReceiver, new IntentFilter("GPSLocationUpdates"));
    mStatus = "recording";
    startServiceAndBind();
    result.success(null);
  }
  private BroadcastReceiver mMessageReceiver = new BroadcastReceiver() {
    @Override
    public void onReceive(Context context, Intent intent) {
      // Get extra data included in the Intent
      HashMap<String, Object> hashMap = (HashMap<String, Object>) intent.getSerializableExtra("current");
      qe.add(hashMap);
    }
  };


  private void handlePause(MethodCall call, Result result) {
    mStatus = "paused";
    mAudioRecordService.pauseRecord();

    qe.clear();
    result.success(null);
  }

  private void handleResume(MethodCall call, Result result) {
    mStatus = "recording";
    mAudioRecordService.resumeRecord();

    result.success(null);
  }

  private void handleStop(MethodCall call, Result result) {
    if(mStatus.equals("stopped")) {
      result.success(null);
    } else {
      mStatus = "stopped";

      // Return Recording Object
      HashMap<String, Object> currentResult = new HashMap<>();
      currentResult.put("duration", 0);
      currentResult.put("path", mFilePath);
      currentResult.put("audioFormat", mExtension);
      currentResult.put("peakPower", -120.00);
      currentResult.put("averagePower", -120.00);
      currentResult.put("isMeteringEnabled", true);
      currentResult.put("status", mStatus);
      currentResult.put("buffer", tempByteBuffer);


      qe.clear();
      stopServiceAndUnBind();

      result.success(currentResult);
    }

  }













  private boolean mIsServiceBound = false;
  private AudioRecordService mAudioRecordService;

  private final ServiceConnection serviceConnection = new ServiceConnection() {
    @Override public void onServiceConnected(ComponentName componentName, IBinder iBinder) {
      mIsServiceBound = true;
      mAudioRecordService =
              ((AudioRecordService.ServiceBinder) iBinder).getService();
      Log.i("Tesing", " " + mAudioRecordService.isRecording() + " recording");
    }

    @Override public void onServiceDisconnected(ComponentName componentName) {
    }
  };

  public void startServiceAndBind() {
    Log.d("kiemtra","startServiceAndBind");
    intent.putExtra("mFilePath",mFilePath);
    intent.putExtra("mSampleRate",mSampleRate);
    registrar.context().startService(intent);
    bindToService();
  }


  public void stopServiceAndUnBind() {
    registrar.context().stopService(intent);
    unbindFromService();
  }

  void unbindFromService() {
    unRegisterLocalBroadCastReceiver();
    if (mIsServiceBound) {
      mIsServiceBound = false;
      registrar.context().unbindService(serviceConnection);
    }
  }
  private void unRegisterLocalBroadCastReceiver() {
    LocalBroadcastManager.getInstance(registrar.context()).unregisterReceiver(serviceUpdateReceiver);
  }
  void bindToService() {
    Log.d("kiemtra","bindToService");
    registrar.context().bindService(intent, serviceConnection, Context.BIND_AUTO_CREATE);
    registerLocalBroadCastReceiver();
  }
  private void registerLocalBroadCastReceiver() {
    LocalBroadcastManager.getInstance(registrar.context())
            .registerReceiver(serviceUpdateReceiver, new IntentFilter(AppConstants.ACTION_IN_SERVICE));
  }
  private final BroadcastReceiver serviceUpdateReceiver = new BroadcastReceiver() {
    @Override public void onReceive(Context context, Intent intent) {
      if (!intent.hasExtra(AppConstants.ACTION_IN_SERVICE)) return;
      String actionExtra = intent.getStringExtra(AppConstants.ACTION_IN_SERVICE);
    }
  };


}
