import 'dart:async';
import 'dart:io';
import 'package:grpc/grpc.dart';
import '../proto/generated/cloud_speech.pbgrpc.dart' as cloudSpeech;
import '../vais_speech_audio.dart';

class VaisStreamingController {
  cloudSpeech.StreamingRecognizeRequest gRpcRequestStream;
  cloudSpeech.StreamingRecognitionConfig gRpcConfigStream;
  cloudSpeech.RecognitionConfig gRpcConfig;
  cloudSpeech.SpeechClient gRpcStub;
  String _token;
  StreamController<cloudSpeech.StreamingRecognizeRequest> gRpcRequestStreamController = StreamController();

  String get token => _token;

  VaisStreamingController(this._token);

  void initialize() {
    final channel = ClientChannel('asr-benchmark.vais.vn',
        port: 50050,
        options: const ChannelOptions(credentials: const ChannelCredentials.insecure()));
    gRpcStub = cloudSpeech.SpeechClient(channel, options: CallOptions(metadata: {"api-key" : _token}));
    gRpcConfig = cloudSpeech.RecognitionConfig();
    gRpcConfigStream = cloudSpeech.StreamingRecognitionConfig.create();
    gRpcRequestStream = cloudSpeech.StreamingRecognizeRequest.create();
  }

  void dispose() {
    gRpcRequestStreamController?.close();
  }

  void requestBeforeStartAudioRecord(bool singleUtterance, VaisSpeechCallback listener,String modelParamText) {
    gRpcConfig.clear();
    gRpcConfig.encoding = cloudSpeech.RecognitionConfig_AudioEncoding.LINEAR16;

    gRpcConfig.sampleRateHertz = Platform.isIOS ? 22050 : 16000;
    var modelParam = new cloudSpeech.ModelParam();
    modelParam.graph = modelParamText;
    gRpcConfig.modelParam = modelParam;

    gRpcConfigStream.clear();
    gRpcConfigStream.config = gRpcConfig;
    gRpcConfigStream.interimResults = true;
    gRpcConfigStream.singleUtterance = singleUtterance;

    gRpcRequestStream.clear();
    gRpcRequestStream.streamingConfig = gRpcConfigStream;
    if (gRpcRequestStreamController.isClosed) {
      gRpcRequestStreamController = StreamController();
    }
    gRpcRequestStreamController.sink.add(gRpcRequestStream);

    gRpcStub.streamingRecognize(gRpcRequestStreamController.stream).listen((data) {
      var result = data.results[0];
      final String output = result.alternatives[0].transcript;
      print("Success >>>>>>>>>>>>>> $output");
      print(gRpcConfig.modelParam.graph.toString());

      listener.onConnectionChanged(true);
      listener.onGeneratedSpeechText(output);
    }, onError: (error) {
      print("Error $error");
      listener.onConnectionChanged(false);
    }, onDone: () {
      print("Done");
      listener.onConnectionChanged(false);
    });
  }

  void streamingAudioRecord(List<int> buffers) {
    gRpcRequestStream.clear();
    gRpcRequestStream.audioContent = buffers;
    gRpcRequestStreamController.add(gRpcRequestStream);
  }

  bool isStreamingClosed() {
    return gRpcRequestStreamController?.isClosed ?? true;
  }
}